#//-----------------------------------------------
#//TUSB-0224ADM用Pythonサンプルプログラム
#// 処理関数等
#//
#//2024_04_08　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

import tkinter as tk
import tkinter.ttk as ttk
from tkinter import messagebox

def ShowInfo(str):
    """文字列を表示する"""
    messagebox.showinfo("",str)

def ForceToInt(nstr,err=-1):
    """文字列をintで返す。他の型でも可能な限りintに変換して返す。変換できなくても何か返す。
    
    Arguments:
    str         ----    変換したい文字列
    err         ----    変換できない時に返す値(デフォルト-1)
    
    Return:
    int         ----    変換した値またはエラー指定値
    """
    if type(nstr) is int: return nstr
    if type(nstr) is float:
        try:
            return int(nstr)
        except Exception:
            return err
    if (type(nstr) is str):
        try:
            if nstr.isdecimal() : return int(nstr)
            return int(float(nstr))
        except Exception:
            return err
    return err

def bitChk(dat: int,bit) -> bool:
    """datを2進数としてbit桁目のビットをチェックする

    Reuturns:
    bool            ----    ビットが1ならTrue 0ならFalse
    """
    dat >>= bit
    return (( dat & 1 ) == 1)
    
BaseClkDict = {
    0:512000.0,
    1:256000.0,
    2:128000.0,
    3:64000.0,
    4:400000.0,
    5:200000.0,
    6:100000.0,
    7:50000.0,
    8:11291676.0/32.0,
    9:11291676.0/64.0,
    10:11291676.0/128.0,
    11:11291676.0/256.0,
    12:384000.0,
    13:192000.0,
    14:96000.0,
    15:48000.0,
}
def GenSampleRateStr(src,div):
    if src >= 16:
        divr = 32 * 2 ** (src-16)
        return ("外部クロック/"+str(divr))
    clksps = BaseClkDict[src] / div
    if clksps >= 100.0E3:
        return '{:.3f}'.format(clksps/1000.0)+"kSPS"
    elif clksps >= 10.0E3:
        return '{:.3f}'.format(clksps/1000.0)+"kSPS"
    elif clksps >= 1.0E3:
        return '{:.4f}'.format(clksps/1000.0)+"kSPS"
    else:
        return  '{:.3f}'.format(clksps)+"SPS"
    
def GetLFrame(master,title,relw,relh,relx,rely):
    """ラベル付きフレームを作成する
    
    arguments:
    master      ----    配置ペアレント
    title       ----    フレームのタイトル文字列
    relw        ----    表示幅(相対値)
    relh        ----    表示高(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上端)

    returns:
    LabelFrame  ----    生成されたラベル付きフレームインスタンス
    """
    frame = tk.LabelFrame(master, text=title)
    frame.place(relwidth= relw,relheight=relh,relx=relx,rely=rely,anchor=tk.NW)
    return frame

def GetLabel(master,text,relx,rely):
    """ラベルを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    表示する文字列
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Label       ----    生成されたラベルインスタンス
    """
    label = tk.Label(master,text=text)
    label.place(relx=relx,rely=rely,anchor=tk.W)
    return label

def GetEntry(master,init,relw,relx,rely):
    """テキスト入力ボックスを作成する
    
    arguments:
    master      ----    配置ペアレント
    init        ----    初期入力文字列
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Entry       ----    生成されたテキストボックスインスタンス
    """
    entry = tk.Entry(master)
    entry.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    entry.insert(tk.END,init)
    return entry

def GetButton(master,text,command,relw,relx,rely):
    """ボタンを作成する
    
    arguments:
    master      ----    配置ペアレント
    text        ----    ボタンに表示する文字列
    command     ----    ボタンが押された時に呼び出されるハンドラ関数
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左右中央)
    rely        ----    Y座標相対位置(上下中央)

    returns:
    Button      ----    生成されたボタンインスタンス
    """
    button = tk.Button(master,text=text,command=command)
    button.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.CENTER)
    return button

def GetComboBox(master,value,relw,relx,rely,command=None):
    """コンボボックスを作成する
    
    arguments:
    master      ----    配置ペアレント
    value       ----    コンボボックスに設定するデータリスト(又はタプル)
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    command     ----    選択値が変更した時に呼び出される関数(引数としてイベントが渡される)

    returns:
    Combobox    ----    生成されたコンボボックスインスタンス
    """
    combobox = ttk.Combobox(master,value=value,state="readonly")
    combobox.bind("<<ComboboxSelected>>",command)
    combobox.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    combobox.current(0)
    return combobox

def GetRadioButtons(master,dic,relx,rely,relxp,relyp,val,command=None):
    """ラジオボタン選択を作成する

    arguments:
    master      ----    配置ペアレント
    dic         ----    ラジオボタンのtextをkey、ラジオボタンのvalueをvalueにセットした辞書型
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    relxp       ----    ラジオボタン間左右ピッチ
    relyp       ----    ラジオボタン間上下ピッチ
    val         ----    variableを対応させるwidget変数(例 IntVar())
    command     ----    ラジオボタン変更時ハンドラ関数

    returns:
    ラジオボタンインスタンスリスト
    """
    ret_rbtn = list()
    cnt = 0
    for key in dic.keys():
        ret_rbtn.append(tk.Radiobutton(master,text=key,value=dic[key],variable=val,command=command))
        ret_rbtn[cnt].place(relx = relx+cnt*relxp,rely = rely+cnt*relyp,anchor=tk.W)
        cnt += 1
    return ret_rbtn

def GetSpinBoxN(master,min,max,inc,val,relw,relx,rely,command=None):
    """整数数値入力スピンボックスを生成する
    
    arguments:
    master      ----    配置ペアレント
    min         ----    数値の最小値
    max         ----    数値の最大値
    inc         ----    上下スピンボタン押下時の変化量
    val         ----    variableを対応させるwidget変数(例 IntVar())
    relw        ----    表示幅(相対値)
    relx        ----    X座標相対位置(左端)
    rely        ----    Y座標相対位置(上下中央)
    command     ----    値変更時ハンドラ関数

    returns:
    Spinbox     ----    生成された入力スピンボックスのインスタンス
    """
    spinbox = tk.Spinbox(master,from_=min,to=max,increment=inc,textvariable=val,command=command)
    spinbox.place(relw=relw,relx=relx,rely=rely,anchor=tk.W)
    return spinbox

def GetLabelDisp(master,text,relw,relx,rely):
    label = tk.Label(master,text=text,relief="groove")
    label.place(relwidth=relw,relx=relx,rely=rely,anchor=tk.W)
    return label
def GetChkBox(master,text,relx,rely,val):
    chkbox = tk.Checkbutton(master,text=text,variable=val)
    chkbox.place(relx=relx,rely=rely,anchor=tk.W)
    return chkbox
def GetLedCanvas(master,color,relx,rely):
    canvas = tk.Canvas(master,width=15,height=15,bg=color)
    canvas.place(relx=relx,rely=rely,anchor = tk.CENTER)
    return canvas
def GetLabelDispML(master,text,relw,relh,relx,rely):
    label = tk.Label(master,text=text,relief="groove")
    label.place(relwidth=relw,relheight=relh,relx=relx,rely=rely,anchor=tk.NW)
    return label