#//-----------------------------------------------
#//TUSB-0224ADM用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_04_08　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#Device DLL Loader windll -> sdtcall
tusb0224admdll = windll.LoadLibrary("TUSB0224ADM.DLL")  

#Function definitions of DLL driver API  (don not call directry from user codes)
tusb0224admdll.TUSB0224ADM_Connect.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Connect.argtypes = (c_char_p,)
tusb0224admdll.TUSB0224ADM_DisConnect.restype = c_uint32
tusb0224admdll.TUSB0224ADM_DisConnect.argtypes = (c_char_p,)
tusb0224admdll.TUSB0224ADM_Calib_Val.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Calib_Val.argtypes = (c_char_p,c_ubyte,POINTER(c_int),POINTER(c_double))
tusb0224admdll.TUSB0224ADM_Dio_In.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Dio_In.argtypes = (c_char_p,POINTER(c_ubyte))
tusb0224admdll.TUSB0224ADM_Dio_Out.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Dio_Out.argtypes = (c_char_p,c_ubyte)
tusb0224admdll.TUSB0224ADM_SyncMode.restype = c_uint32
tusb0224admdll.TUSB0224ADM_SyncMode.argtypes = (c_char_p,c_ubyte)
tusb0224admdll.TUSB0224ADM_SyncAdj.restype = c_uint32
tusb0224admdll.TUSB0224ADM_SyncAdj.argtypes = (c_char_p,c_ubyte)
tusb0224admdll.TUSB0224ADM_Ad_Sample.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Ad_Sample.argtypes = (c_char_p,POINTER(c_int))
tusb0224admdll.TUSB0224ADM_Ad_IntClock.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Ad_IntClock.argtypes = (c_char_p,c_ubyte,c_uint,c_ubyte)
tusb0224admdll.TUSB0224ADM_Ad_Input.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Ad_Input.argtypes = (c_char_p,c_ubyte,c_ubyte,c_ubyte,c_ubyte)
tusb0224admdll.TUSB0224ADM_Ad_InputChk.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Ad_InputChk.argtypes = (c_char_p,c_ubyte,POINTER(c_ubyte),POINTER(c_ubyte),POINTER(c_ubyte))
tusb0224admdll.TUSB0224ADM_Smpl_TrgLevel.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Smpl_TrgLevel.argtypes = (c_char_p,c_ubyte,c_ubyte,c_int,c_int)
tusb0224admdll.TUSB0224ADM_Smpl_Start.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Smpl_Start.argtypes = (c_char_p,c_uint,c_ubyte)
tusb0224admdll.TUSB0224ADM_Smpl_Stop.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Smpl_Stop.argtypes = (c_char_p,)
tusb0224admdll.TUSB0224ADM_Smpl_Status.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Smpl_Status.argtypes = (c_char_p,c_ubyte,POINTER(c_ubyte),POINTER(c_uint))
tusb0224admdll.TUSB0224ADM_Smpl_Data.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Smpl_Data.argtypes = (c_char_p,c_ubyte,POINTER(c_int),POINTER(c_uint))
tusb0224admdll.TUSB0224ADM_Smpl_SoftTrigger.restype = c_uint32
tusb0224admdll.TUSB0224ADM_Smpl_SoftTrigger.argtypes = (c_char_p,)


#######  support functions (don not call directry from user codes)
def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):   return False
    elif(Val < Min):            return False
    elif(Val > Max):            return False
    else:                       return True

def IsValidSN(SerialN):
    return IsInRange(SerialN,0,9999)

######　definitions of Function
#SerialNumber Device Serial Number 0~9999

#definitions
c_uint32_max = 2 ** 32 - 1

#serialnumber string dictionary
SerStr = dict()     # Serial Number strings storage dictionary

#def functions

def TUSB0224ADM_Connect(SerialNumber):
    """Connect Device

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999

    Returns:
    int             ----    Error Code
    """
    global SerStr
    if( IsValidSN(SerialNumber) ):
        SerStr[SerialNumber] = c_char_p(format(SerialNumber,'04').encode('utf-8'))
        return tusb0224admdll.TUSB0224ADM_Connect(SerStr[SerialNumber])
    return 1

def TUSB0224ADM_DisConnect(SerialNumber):
    """Disconnect Device

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999

    Returns:
    int             ----    Error Code
    """
    if( SerialNumber in SerStr ):
        return tusb0224admdll.TUSB0224ADM_DisConnect(SerStr[SerialNumber])
    return 5

def TUSB0224ADM_Calib_Val(SerialNumber,Ch):
    """Get Calibration Values

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    Ch              ----    Target Channel

    Returns:
    int,            ----    Error Code
    (int * 9),      ----    Offset tuple
    (double * 6),   ----    Lsb Voltage tuple
    """
    if( SerialNumber in SerStr ):
        if( not IsInRange(Ch,0,1) ): return 1
        tmpOffset = (c_int * 9)()     # 9 c_uint instance
        p_tmpOffset = cast(pointer(tmpOffset),POINTER(c_int))  #pointer of tmpOffset
        tmpLsbv = (c_double * 6 )()         # 6 c_double instance
        p_tmpLsbv = cast(pointer(tmpLsbv),POINTER(c_double))    #pointer of tmpLsbv
        retv = tusb0224admdll.TUSB0224ADM_Calib_Val(SerStr[SerialNumber],Ch,p_tmpOffset,p_tmpLsbv)
        return retv,tuple(tmpOffset),tuple(tmpLsbv)
    return 5,None,None

def TUSB0224ADM_Dio_In(SerialNumber):
    """Read Digital Inputs

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999

    Returns:
    int,            ----    Error Code
    int,            ----    Read data
    """
    if( SerialNumber in SerStr ):
        tmpDat = c_ubyte()
        retv = tusb0224admdll.TUSB0224ADM_Dio_In(SerStr[SerialNumber],byref(tmpDat))
        return retv,tmpDat.value
    return 5,None

def TUSB0224ADM_Dio_Out(SerialNumber,OutData):
    """Set Digital Output

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    OutData         ----    Output Data

    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr ):
        if( not IsInRange(OutData,0,15) ): return 1
        return tusb0224admdll.TUSB0224ADM_Dio_Out(SerStr[SerialNumber],OutData)
    return 5
    
def TUSB0224ADM_SyncMode(SerialNumber,Mode):
    """Set Synchronous Mode

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    Mode            ----    Synchronous mode

    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr ):
        if( not IsInRange(Mode,0,2) ): return 1
        return tusb0224admdll.TUSB0224ADM_SyncMode(SerStr[SerialNumber],Mode)
    return 5
    
def TUSB0224ADM_SyncAdj(SerialNumber,Com):
    """Adjust Synchronous Function

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    Com             ----    Synchronous type

    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr):
        if( not IsInRange(Com,0,2) ): return 1
        return tusb0224admdll.TUSB0224ADM_SyncAdj(SerStr[SerialNumber],Com)
    return 5

def TUSB0224ADM_Ad_Sample(SerialNumber):
    """Get Converted A/D Datas

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999

    Returns:
    int,            ----    Error Code
    int,            ----    Conversion Data of ch1
    int,            ----    Conversion Data of ch2
    """
    if( SerialNumber in SerStr):
        tmpData = (c_int * 2)()     #c_uint を9個持つ型のインスタンス
        p_tmpData = cast(pointer(tmpData),POINTER(c_int))  #上記のインスタンスのポインタ
        retv = tusb0224admdll.TUSB0224ADM_Ad_Sample(SerStr[SerialNumber],p_tmpData)
        return retv,tmpData[0],tmpData[1]
    return 5,None,None

def TUSB0224ADM_Ad_IntClock(SerialNumber,Sel,ClkDiv,Filter):
    """Set continuuous Acquisition clock

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    Sel             ----    Base clock Selection
    ClkDiv          ----    Division Count
    Filter          ----    LP Filter Selection
    
    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr):
        if( not IsInRange(Sel,0,19)): return 1
        if( not IsInRange(ClkDiv,0,256)): return 1
        if( not IsInRange(Filter,0,1)): return 1
        return tusb0224admdll.TUSB0224ADM_Ad_IntClock(SerStr[SerialNumber],Sel,ClkDiv,Filter)
    return 5
    
def TUSB0224ADM_Ad_Input(SerialNumber, Ch, Sens, Couple, Gain):
    """Set Analog input

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    Ch              ----    Target Channel
    Sens            ----    Attenuator Selection
    Couple          ----    AC/DC Couple Selection
    Gain            ----    Amp. Gain Selection

    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr):
        if( not IsInRange(Ch,0,1)): return 1
        if( not IsInRange(Sens,0,1)): return 1
        if( not IsInRange(Couple,0,1)): return 1
        if( not IsInRange(Gain,0,2)): return 1
        return tusb0224admdll.TUSB0224ADM_Ad_Input(SerStr[SerialNumber],Ch,Sens,Couple,Gain)
    return 5

def TUSB0224ADM_Ad_InputChk(SerialNumber,Ch):
    """Check Analog input
    
    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999

    Returns:
    int,            ----    Error Code
    int,            ----    Attenuator Selection
    int,            ----    AC/DC Couple Selection
    int,            ----    Amp. Gain Selection
    """
    if( SerialNumber in SerStr ):
        if( not IsInRange(Ch,0,1)): return 1
        Sens = c_ubyte()
        Couple = c_ubyte()
        Gain = c_ubyte()
        retv = tusb0224admdll.TUSB0224ADM_Ad_InputChk(SerStr[SerialNumber],Ch,byref(Sens),byref(Couple),byref(Gain))
        return retv,Sens.value,Couple.value,Gain.value
    return 5,None,None,None

def TUSB0224ADM_Smpl_TrgLevel(SerialNumber,Select,AinSel,Level1,Level2):
    """Set Trigger settings

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    Select          ----    Trigger Type Selection
    AinSel          ----    Trigger Channel Selection
    Level1          ----    Trigger Level 1
    Level2          ----    Trigger Level 2

    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr ):
        if( not IsInRange(Select,0,6)): return 1
        if( not IsInRange(AinSel,0,1)): return 1
        if( not IsInRange(Level1,-0x800000,+0x7fffff)): return 1
        if( not IsInRange(Level2,-0x800000,+0x7fffff)): return 1
        return tusb0224admdll.TUSB0224ADM_Smpl_TrgLevel(SerStr[SerialNumber],Select,AinSel,Level1,Level2)
    return 5

def TUSB0224ADM_Smpl_Start(SerialNumber,PreTrgLen,SampleChSel):
    """Start Continuous Sampling

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    PreTrgLen       ----    Pre Trigger Length
    SampleClkChSel  ----    Sampling Channel Selection

    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr ):
        if( not IsInRange(PreTrgLen,0,1048576)): return 1
        if( not IsInRange(SampleChSel,0,2)): return 1
        return tusb0224admdll.TUSB0224ADM_Smpl_Start(SerStr[SerialNumber],PreTrgLen,SampleChSel)
    return 5

def TUSB0224ADM_Smpl_Stop(SerialNumber):
    """Stop Continuous Sampling

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999

    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr ):
        return tusb0224admdll.TUSB0224ADM_Smpl_Stop(SerStr[SerialNumber])
    return 5

def TUSB0224ADM_Smpl_Status(SerialNumber, Ch):
    """Get Continuous Sampling Status

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999

    Returns:
    int,            ----    Error Code
    int,            ----    Status Code
    int,            ----    Number of Sampled data
    """
    if( SerialNumber in SerStr):
        if( not IsInRange(Ch,0,1)): return 1
        Status = c_ubyte()
        DataLen = c_uint()
        retv = tusb0224admdll.TUSB0224ADM_Smpl_Status(SerStr[SerialNumber],Ch,byref(Status),byref(DataLen))
        return retv,Status.value,DataLen.value
    return 5,None,None

def TUSB0224ADM_Smpl_Data(SerialNumber, Ch, DataLen ):
    """Upload Sampled Datas

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999
    Ch              ----    Target Channel
    DataLen         ----    Number of data required

    Returns:
    int,            ----    Error Code
    (int,...)       ----    Sampled Datas (tuple)
    """
    if( SerialNumber in SerStr):
        if( not IsInRange(Ch,0,1)): return 1
        if( not IsInRange(DataLen,0,c_uint32_max)): return 1
        reqDataLen = c_uint(DataLen)
        Data = (c_int * DataLen)()
        p_Data = cast(pointer(Data),POINTER(c_int))
        retv = tusb0224admdll.TUSB0224ADM_Smpl_Data(SerStr[SerialNumber],Ch,p_Data,byref(reqDataLen))
        if retv != 0 : return retv,None
        return 0,tuple(Data[:reqDataLen.value])
    return 5,None
    
def TUSB0224ADM_Smpl_SoftTrigger(SerialNumber):
    """Generate a software trigger

    Arguments:
    SerialNumber    ----    Device Serial Number 0~9999

    Returns:
    int,            ----    Error Code
    """
    if( SerialNumber in SerStr):
        return tusb0224admdll.TUSB0224ADM_Smpl_SoftTrigger(SerStr[SerialNumber])
    return 5

ErrStrDic = {
            0:"正常終了しました",
            1:"パラメータエラーです",
            2:"デバイスが接続されていません",
            3:"接続デバイスが多すぎます",
            4:"デバイスが見つかりません",
            5:"USB通信エラーです",
            6:"デバイスは既に接続されています",
            7:"連続取り込み動作中です",
            99:"その他のエラーです"
}   # Return code to Message string dictionary
def GetErrMessage(retcode):
    """Return Error String by return code

    Arguments:
    retcode         ----    Return code

    Returns:
    str,            ----    Return Message String
    """
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    return "不明なエラーです"
    
