"""
-----------------------------------------------
TUSB-0224ADM用Pythonサンプルプログラム
 ドライバアクセスクラス

2024_04_08　株式会社タートル工業
Copyright (C) 2024 Turtle Industry Co.,Ltd.
-----------------------------------------------
"""

import TUSB0224ADM  # TUSB-0224ADM ドライバ呼び出しモジュール
import threading

class TUSB0224ADM_Mod():
    """TUSB0224ADM Control Class
    """
    def GetInvDic(self,dic: dict) -> dict:
        """Key and Value swap converter
        
        Arguments:
        dic         ----    source dictionary

        Returns:
        dict        ----    converted result dictionary
        """
        return {val:key for key, val in dic.items()} 
    
    
    
    # Dictionaries for Slection valus

    # Ad Channel
    dicAdCh = {
        0:"Ch1",
        1:"Ch2"
    }
    dicnAdCh = GetInvDic(None,dicAdCh)

    # Attenurator
    dicAttn = {
        0:"/1",
        1:"/10"
    }
    dicnAttn = GetInvDic(None,dicAttn)

    # Input Couple
    dicCouple = {
        0:"DC",
        1:"AC"
    }
    dicnCouple = GetInvDic(None,dicCouple)

    # Gain
    dicGain = {
        0:"*1",
        1:"*3",
        2:"*10"
    }
    dicnGain = GetInvDic(None,dicGain)

    # Calibration table and functions
    # Calibration table
    OffsetDC = [[[None,None,None],[None,None,None]],[[None,None,None],[None,None,None]]]
    OffsetAC = [[None,None,None],[None,None,None]]
    LsbV = [[[None,None,None],[None,None,None]],[[None,None,None],[None,None,None]]]
    
    def SetCaliTable(self,ch,offset,lsbv):
        """Set Calibration data to tables
        
        Aurguments:
        self            ----    
        ch              ----    Channel 0 or 1
        offset          ----    tuple(or list) of offset data 
        lsbv            ----    tuple(or list) of lsb voltage data
        """
        self.OffsetDC[ch][0][0] = offset[0] # OffsetDC[ch][attn][gain]
        self.OffsetDC[ch][0][1] = offset[1]
        self.OffsetDC[ch][0][2] = offset[2]
        self.OffsetDC[ch][1][0] = offset[3]
        self.OffsetDC[ch][1][1] = offset[4]
        self.OffsetDC[ch][1][2] = offset[5]
        self.OffsetAC[ch][0] = offset[6]    # OffsetAC[ch][gain]
        self.OffsetAC[ch][1] = offset[7]
        self.OffsetAC[ch][2] = offset[8]
        self.LsbV[ch][0][0] = lsbv[0]       # LsbV[ch][attn][gain]
        self.LsbV[ch][0][1] = lsbv[1]
        self.LsbV[ch][0][2] = lsbv[2]
        self.LsbV[ch][1][0] = lsbv[3]
        self.LsbV[ch][1][1] = lsbv[4]
        self.LsbV[ch][1][2] = lsbv[5]
        
    def ApplyCalib(self,data,ch):
        """apply calibration table to ad rawdata
        
        Arguments:
        self        ----    
        data        ----    adc raw data
        ch          ----    channel 0 or 1

        Returns:
        float       ----    result voltage data
        """
        if self.Couple[ch]== self.dicnCouple["DC"]:
            # DC
            offset = self.OffsetDC[ch][self.Attn[ch]][self.Gain[ch]]
        else:
            # AC
            offset = self.OffsetAC[ch][self.Gain[ch]]
        lsbv = self.LsbV[ch][self.Attn[ch]][self.Gain[ch]]
        volt = (data- offset) *lsbv
        return volt
    
    # Ad Input Setting
    Attn = [None,None]
    Couple = [None,None]
    Gain = [None,None]   

    # synchronization lock
    lock = threading.Lock()

    def __init__(self,SerialNum):
        """Initializer

        Arguments:
        self        ----    
        SerialNum   ----    Device Serial Number 0 ~ 9999

        Returns:
        None
        """
        ret = TUSB0224ADM.TUSB0224ADM_Connect(SerialNum)
        self.SerialNum = SerialNum 
        if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))
        for ch in range(2):
            ret = TUSB0224ADM.TUSB0224ADM_Calib_Val(SerialNum,ch)
            if ret[0] != 0:
                TUSB0224ADM.TUSB0224ADM_DisConnect(SerialNum)
                raise Exception(TUSB0224ADM.GetErrMessage(ret[0]))
            self.SetCaliTable(ch,ret[1],ret[2])
            [self.Attn[ch],self.Couple[ch],self.Gain[ch]] = self.Ad_InputChk(ch)
    
    def DevClose(self):
        """Finalizer

        Arguments:
        self        ----    

        Returns:
        None
        """
        if( self.SerialNum != None ):
            if self.lock.acquire(timeout=1):
                try:
                    TUSB0224ADM.TUSB0224ADM_DisConnect(self.SerialNum)
                except Exception as e:
                    print(e)
                self.lock.release()
            self.SerialNum = None

    def Dio_In(self):
        """Digital Input
        
        Returns:
        int         ----    ReadData
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Dio_In(self.SerialNum)
                self.lock.release()
            if(ret[0]!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret[0]))
            return ret[1]
        
    def Dio_Out(self,OutData):
        """Digital Output
        Arguments:
        self        ----
        OutData     ----    Output Data
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Dio_Out(self.SerialNum,OutData)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))

    def SyncMode(self,mode):
        """Set Synchronous Mode
        """
        if( self.SerialNum != None ):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_SyncMode(self.SerialNum,mode)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))
    
    def SyncAdj(self,com):
        """Sync Adjust
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_SyncAdj(self.SerialNum,com)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))

    def Ad_Sample(self):
        """Get one Sampled data set

        Returns:
        float           ----    Ch1 Voltage
        float           ----    Ch2 Voltage
        """
        if( self.SerialNum != None ):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Ad_Sample(self.SerialNum)
                self.lock.release()
            if(ret[0]!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))
            VoltCh1 = self.ApplyCalib(ret[1],0)
            VoltCh2 = self.ApplyCalib(ret[2],1)
            return VoltCh1,VoltCh2

    def Ad_IntClock(self,Sel,ClkDiv,Filter):
        """Set Internal Clock

        Arguments:
        Sel             ----    Base clock Selection
        ClkDiv          ----    Division Count
        Filter          ----    LP Filter Selection
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Ad_IntClock(self.SerialNum,Sel,ClkDiv,Filter)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))

    def Ad_Input(self, Ch, Attn, Couple, Gain):
        """Set Ad Input

        Arguments:
        Ch              ----    Ch Select 0:Ch1 1:Ch2
        Attn            ----    Attenuator Select 0:
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Ad_Input(self.SerialNum,Ch,Attn,Couple,Gain)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))
            self.Attn[Ch] = Attn            #Attenuator Selection
            self.Couple[Ch] = Couple        #AC/DC Couple Selection
            self.Gain[Ch] = Gain            #Amp. Gain Selection
    
    def Ad_InputChk(self, Ch):
        """Check Ad Input

        Arguments:
        Ch              ----    Ch Select 0:Ch1 1:Ch2

        Returns:
        int             ----    Attenuator Selection 0:1/1  1:1/10
        int             ----    AC / DC Couple Selection
        int             ----    Amp. Gain Selection
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Ad_InputChk(self.SerialNum,Ch)
                self.lock.release()
            if(ret[0]!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))
            return (ret[1],ret[2],ret[3])
    
    def Smpl_TrgLevel(self,Select,AinSel,Level1,Level2):
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Smpl_TrgLevel(self.SerialNum,Select,AinSel,Level1,Level2)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))
    
    def Smpl_Start(self,PreTrgLen,SampleChSel):
        """Start Continuous Sampling

        Arguments:
        PreTrgLen       ----    Pre Trigger Length
        SampleClkChSel  ----    Sampling Channel Selection
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Smpl_Start(self.SerialNum,PreTrgLen,SampleChSel)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))
    
    def Smpl_Stop(self):
        """Stop Continuous Sampling
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Smpl_Stop(self.SerialNum)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))
    
    def Smpl_Status(self,Ch):
        """Get Continuous Sampling Status

        Returns:
        int,            ----    Status Code
        int,            ----    Number of Sampled data
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Smpl_Status(self.SerialNum,Ch)
                self.lock.release()
            if(ret[0]!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret[0]))
            return ret[1],ret[2]

    def Smpl_Data(self,Ch,DataLen):
        """Upload Sampled Datas
        Arguments:
        Ch              ----    Target Channel
        DataLen         ----    Number of data required
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Smpl_Data(self.SerialNum,Ch,DataLen)
                self.lock.release()
            if(ret[0]!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret[0]))
            voltList = list()
            for data in ret[1]:
                voltList.append(self.ApplyCalib(data,Ch))
            return voltList

    def Smpl_SoftTrigger(self):
        """Generate a software trigger
        """
        if( self.SerialNum != None):
            if self.lock.acquire(timeout=1):
                ret = TUSB0224ADM.TUSB0224ADM_Smpl_SoftTrigger(self.SerialNum)
                self.lock.release()
            if(ret!=0):raise Exception(TUSB0224ADM.GetErrMessage(ret))

